SuperSakura engine
------------------

SuperSakura is a modern visual novel engine that can run certain Japanese
games from the mid-90's. Many of these old titles were surprisingly good,
but were never localised. SuperSakura has tools to help localise games, and
supports enhanced graphics and a modernised user interface.

The engine is written in Free Pascal, uses SDL2, and targets Linux/Windows,
64/32-bit.

Note that this is just a game engine and a set of asset conversion tools.
The actual games themselves are under copyright and are not distributed with
this project. To run games on SuperSakura, you need to convert the game data
from original files.

For a list of supported games, run `decomp -list`, or see
`inc/gidtable.inc`, or visit the main site at
[mooncore.eu/ssakura](https://mooncore.eu/ssakura/).

To see how the engine is progressing, see [CHANGELOG.md](CHANGELOG.md) and
[todo.md](doc/todo.md).


### Screenshots

![Front end screenshot](https://mooncore.eu/ssakura/gfx/sakutitle.png "Front end screenshot")

For more, see [here](https://mooncore.eu/ssakura/sscreens.php).


### Downloads

You can get a reasonably recent Win32 build of the engine and tools from
[mooncore.eu/ssakura](https://mooncore.eu/ssakura/). But for best results,
see below on how to compile your own copy.

You MUST have SDL2 runtimes installed to run the engine!


### SDL2

This is a famous video/audio library used by many modern games, so you may
already have it on your system. If not, it is easy to get.

**32-bit Windows**: Get the 32-bit runtime binary for Windows from
[libsdl.org/download-2.0.php](https://libsdl.org/download-2.0.php), and the
font-rendering 32-bit runtime binary from
[libsdl.org/projects/SDL_ttf](https://www.libsdl.org/projects/SDL_ttf/).
Put both in your `\Windows\System32` directory (or in the same directory
where SuperSakura is, if you prefer).

**64-bit Windows**: You can run the 32-bit version of SuperSakura on 64-bit
Windowses, so do the same as above, except you may need to put the DLL files
in your `\Windows\SysWOW64` directory. If you've built a 64-bit version of
SuperSakura, get the 64-bit SDL2 and SDL2_ttf runtime binaries and put those
in your `\Windows\System32` directory (or in the same directory where
SuperSakura is, if you prefer).

**Linux**: Your distro's main software repository should have SDL2 and
SDL2_ttf. On Debian/Ubuntu, the packages may be named libsdl2-2.0-0 and
libsdl2-ttf-2.0-0. Install both through your normal package manager. You
should end up with libSDL2*.so files somewhere under your `/usr/lib` or
`/usr/lib/x86_64-linux-gnu` directory. The two files that must be present
are libSDL2.so and libSDL2_ttf.so. If you do not have these, it is possible
your package manager only set up version-numbered files and neglected to
create generically named links for them. Try to install the development versions
of libsdl2 packages in this case.

Some useful information on setting up SDL2 for Linux is
[here](http://www.freepascal-meets-sdl.net/chapter-2-installation-configuration-linux-version/).
- If SDL has trouble creating a renderer, you may be able to override the
  renderer preference: [Overriding the SDL video driver](https://wiki.libsdl.org/FAQUsingSDL).
  Try something like `SDL_RENDER_DRIVER=software` or `SDL_VIDEODRIVER=x11`.
- If SDL_InitSubSystem throws a division by zero or invalid operation when
  setting up SDL_video, particularly in a virtual machine, see
  [this bug](https://github.com/PascalGameDevelopment/SDL2-for-Pascal/issues/56).
- If video-related failures persist, examine `saku.log` and try running
  `LIBGL_DEBUG=verbose ./supersakura` if on Linux for more information.


### Compiling

Please see [BUILDING.md](/doc/BUILDING.md).


### Using the engine

To compile resources into a usable SuperSakura data file:

    recomp <projectname>

For example, to compile the included Winterquest ministory/testsuite:

    recomp winterq

To run any compiled game through a friendly frontend:

    supersakura

Or, to run the console port:

    supersakura-con

The tools and engine print their log output into files: `saku.log`,
`recomp.log`, and `decomp.log`. By default, these are put in the program's
working directory. If the working directory is write-protected, then the
logs are saved under your profile directory.

You can add -h to any executable's commandline to see what other commandline
options are available.

The console port works best when playing games in English. If your console
is configured to correctly display Japanese characters, you can also play in
Japanese. But successfully configuring a console to show UTF-8-encoded
Japanese, especially in Windows, can be challenging. I was able to get it
working in various Linuxes and Windows XP, but not in Windows 7.


### Converting game data

The resource decompiler tool takes individual files from original games, and
saves them under SuperSakura's data directory in converted standard file
formats. Note, that although some PC-98 games are supported, Decomp cannot
yet extract the individual data files from .HDI or .FDI images, so if you
keep your PC-98 games in those, you will have to first extract the files
from the disk image manually. Possible tools for this:

* Ryo-Cokey's [FIVEC](https://www.pc98.org/project/fivec.html)
(Windows, GUI)
* [EditDisk](https://hp.vector.co.jp/authors/VA013937/editdisk/index_e.html)
(Windows, GUI)
* My own [98ripper](https://gitlab.com/bunnylin/98ripper) (Linux, commandline)

To convert resources:

    decomp <filename or directory>

For example, to convert and run the DOS version of The Three Sisters' Story
(the Windows port is not yet supported):

    decomp /mygames/threesistersstory/
    recomp 3sis
    supersakura 3sis


### Using SakuraScript

If you just want to play the games, you don't need to understand SakuraScript.
This is the custom scripting language used by the engine. The syntax is both
described and verified by the `parsertest` project included in the `data`
directory; you can read the single script file there for details. A list of
recognised built-in functions can be found in the `sakurascript-definition.pas`
source file. Then, if you're curious, you can examine the decompiled scripts
from any supported game and see what's happening behind the scenes.


### Translating games

Under EU law, translations count as derivative works, and it would be
a copyright infringement to make them available to the public without
permission from the rightsholder, whether in patch form or otherwise.
Therefore, translations for the supported games are not distributed with
this project. However, you can easily produce a basic translation through
machine translation, which is legal as long as it is only for your personal
use. Publishing such a translation on peer-to-peer filesharing systems or at
ROMhacking.net would again be infringing.

An automatic translator hook is planned for the engine, but not available yet.
The mcsassm unit has functionality for importing and exporting a game's strings
in simple tab-separated spreadsheets, but this is not currently accessible
through the tools' interface.


### Game modding

You can replace any game file by dropping another file by the same name in the
`data/<project>/override` directory, then restarting the game. This includes
any graphics, scripts, or a translated string table.

For a neater package, you can also create a mod data file. This will show up
as a separate item on the front end. When loaded, the engine loads the parent
dat first, then the mod dat, which replaces files from the parent using the
same override logic as above.

1. Create a new project directory under SuperSakura's data directory, for
example `3sis98-en`.

2. Put your modified files in the new project directory.

3. Create a `data.txt` file there. It should contain a description line, for
example `desc Sanshimai (English mod) (PC98)`, and it should specify the
language: `language English`. Any modified graphics can also be described.

4. Compile the mod by specifying the parent project name, for example:
`recomp 3sis98-en -parent=3sis98`.
